"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.SECRETBOX_NONCE_LENGTH = exports.SECRETBOX_KEY_LENGTH = exports.BOX_SEED_LENGTH = exports.BOX_SECRET_KEY_LENGTH = exports.BOX_PUBLIC_KEY_LENGTH = exports.BOX_NONCE_LENGTH = exports.ARGON2ID_SALTBYTES = exports.ARGON2ID_OPSLIMIT_SENSITIVE = exports.ARGON2ID_OPSLIMIT_MODERATE = exports.ARGON2ID_OPSLIMIT_MIN = exports.ARGON2ID_OPSLIMIT_MAX = exports.ARGON2ID_OPSLIMIT_INTERACTIVE = exports.ARGON2ID_MEMLIMIT_SENSITIVE = exports.ARGON2ID_MEMLIMIT_MODERATE = exports.ARGON2ID_MEMLIMIT_MIN = exports.ARGON2ID_MEMLIMIT_MAX = exports.ARGON2ID_MEMLIMIT_INTERACTIVE = void 0;
exports.aesDecrypt = aesDecrypt;
exports.aesEncrypt = aesEncrypt;
exports.aesGenerateKey = aesGenerateKey;
exports.argon2idDeriveKey = argon2idDeriveKey;
exports.argon2idHash = argon2idHash;
exports.argon2idVerify = argon2idVerify;
exports.boxGenerateKey = boxGenerateKey;
exports.boxOpen = boxOpen;
exports.boxSeal = boxSeal;
exports.decodeBase64 = decodeBase64;
exports.decodeHexadecimal = decodeHexadecimal;
exports.decodeUtf8 = decodeUtf8;
exports.encodeBase64 = encodeBase64;
exports.encodeHexadecimal = encodeHexadecimal;
exports.encodeUtf8 = encodeUtf8;
exports.getRandomBytes = getRandomBytes;
exports.secretboxGenerateKey = secretboxGenerateKey;
exports.secretboxOpen = secretboxOpen;
exports.secretboxSeal = secretboxSeal;
exports.signDetached = signDetached;
exports.signGenerateKey = signGenerateKey;
exports.signVerifyDetached = signVerifyDetached;
var base64 = _interopRequireWildcard(require("@stablelib/base64"));
var hex = _interopRequireWildcard(require("@stablelib/hex"));
var utf8 = _interopRequireWildcard(require("@stablelib/utf8"));
var _hashWasm = require("hash-wasm");
var _tweetnacl = require("tweetnacl");
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
const ARGON2ID_MEMLIMIT_MIN = BigInt(8192);
exports.ARGON2ID_MEMLIMIT_MIN = ARGON2ID_MEMLIMIT_MIN;
const ARGON2ID_MEMLIMIT_MAX = BigInt(4398046510080);
exports.ARGON2ID_MEMLIMIT_MAX = ARGON2ID_MEMLIMIT_MAX;
const ARGON2ID_MEMLIMIT_INTERACTIVE = BigInt(67108864);
exports.ARGON2ID_MEMLIMIT_INTERACTIVE = ARGON2ID_MEMLIMIT_INTERACTIVE;
const ARGON2ID_MEMLIMIT_MODERATE = BigInt(268435456);
exports.ARGON2ID_MEMLIMIT_MODERATE = ARGON2ID_MEMLIMIT_MODERATE;
const ARGON2ID_MEMLIMIT_SENSITIVE = BigInt(1073741824);
exports.ARGON2ID_MEMLIMIT_SENSITIVE = ARGON2ID_MEMLIMIT_SENSITIVE;
const ARGON2ID_OPSLIMIT_MIN = BigInt(1);
exports.ARGON2ID_OPSLIMIT_MIN = ARGON2ID_OPSLIMIT_MIN;
const ARGON2ID_OPSLIMIT_MAX = BigInt(4294967295);
exports.ARGON2ID_OPSLIMIT_MAX = ARGON2ID_OPSLIMIT_MAX;
const ARGON2ID_OPSLIMIT_INTERACTIVE = BigInt(2);
exports.ARGON2ID_OPSLIMIT_INTERACTIVE = ARGON2ID_OPSLIMIT_INTERACTIVE;
const ARGON2ID_OPSLIMIT_MODERATE = BigInt(3);
exports.ARGON2ID_OPSLIMIT_MODERATE = ARGON2ID_OPSLIMIT_MODERATE;
const ARGON2ID_OPSLIMIT_SENSITIVE = BigInt(4);
exports.ARGON2ID_OPSLIMIT_SENSITIVE = ARGON2ID_OPSLIMIT_SENSITIVE;
const ARGON2ID_SALTBYTES = BigInt(16);
exports.ARGON2ID_SALTBYTES = ARGON2ID_SALTBYTES;
const BOX_PUBLIC_KEY_LENGTH = BigInt(_tweetnacl.box.publicKeyLength);
exports.BOX_PUBLIC_KEY_LENGTH = BOX_PUBLIC_KEY_LENGTH;
const BOX_SECRET_KEY_LENGTH = BigInt(_tweetnacl.box.secretKeyLength);
exports.BOX_SECRET_KEY_LENGTH = BOX_SECRET_KEY_LENGTH;
const BOX_NONCE_LENGTH = BigInt(_tweetnacl.box.nonceLength);
exports.BOX_NONCE_LENGTH = BOX_NONCE_LENGTH;
const BOX_SEED_LENGTH = BigInt(32);
exports.BOX_SEED_LENGTH = BOX_SEED_LENGTH;
const SECRETBOX_KEY_LENGTH = BigInt(_tweetnacl.secretbox.keyLength);
exports.SECRETBOX_KEY_LENGTH = SECRETBOX_KEY_LENGTH;
const SECRETBOX_NONCE_LENGTH = BigInt(_tweetnacl.secretbox.nonceLength);

/**
 * Encodes a buffer into a base64 string
 *
 * @throws when the buffer is not an `Uint8Array`
 */
exports.SECRETBOX_NONCE_LENGTH = SECRETBOX_NONCE_LENGTH;
function encodeBase64(buffer) {
  return base64.encode(buffer);
}

/**
 * Decodes a base64 string into a buffer
 *
 * @throws when the given string is not a valid base64 string
 */
function decodeBase64(toEncode) {
  return base64.decode(toEncode);
}

/**
 * Encodes a buffer into an hexadecimal string
 *
 * @throws when buffer is not an `Uint8Array`
 */
function encodeHexadecimal(buffer) {
  return hex.encode(buffer);
}

/**
 * Decodes an hexadecimal string into an Uint8Array
 *
 * @throws when the given string is not a valid hexadecimal string
 */
function decodeHexadecimal(toDecode) {
  return hex.decode(toDecode);
}

/**
 * Encodes a buffer into a UTF-8 string
 *
 * @throws when buffer is not an `Uint8Array`
 */
function encodeUtf8(buffer) {
  return utf8.decode(buffer);
}

/**
 * Decodes a UTF-8 string into an `Uint8Array`
 */
function decodeUtf8(input) {
  return utf8.encode(input);
}

/**
 * Generates a key to be used with aesEncrypt and aesDecrypt
 */
function aesGenerateKey() {
  return (0, _tweetnacl.randomBytes)(32);
}

/**
 * Encrypts a message with AES-256-GCM using the given key
 *
 * @throws when message is not an `Uint8Array`
 * @throws when key is not an `Uint8Array`
 * @throws when key length is incorrect
 */
async function aesEncrypt(message, key) {
  const iv = (0, _tweetnacl.randomBytes)(12);
  const aesKey = await window.crypto.subtle.importKey('raw', key, 'AES-GCM', true, ['encrypt']);
  const encryptedMessage = await window.crypto.subtle.encrypt({
    name: 'AES-GCM',
    iv
  }, aesKey, message);
  return {
    iv,
    encrypted: new Uint8Array(encryptedMessage)
  };
}

/**
 * Decrypts a cipher text with AES-256-GCM using the given key and the initialization vector
 *
 * @throws when cipherText is not an `Uint8Array`
 * @throws when key is not an `Uint8Array`
 * @throws when key length is incorrect
 * @throws when iv is not an `Uint8Array`
 * @throws when iv length is not an `Uint8Array`
 * @throws when the decryption failed
 */
async function aesDecrypt(cipherText, key, iv) {
  const aesKey = await window.crypto.subtle.importKey('raw', key, 'AES-GCM', true, ['encrypt']);
  const decryptedMessage = await window.crypto.subtle.decrypt({
    name: 'AES-GCM',
    iv
  }, aesKey, cipherText);
  return new Uint8Array(decryptedMessage);
}

/**
 * Generates a key pair to be used with boxSeal and boxOpen
 */
function boxGenerateKey() {
  return _tweetnacl.box.keyPair();
}

/**
 * Encrypts a message using the recipient's public key and the sender's secret key
 *
 * @throws when message is not an `Uint8Array`
 * @throws when recipientPublicKey is not an `Uint8Array`
 * @throws when recipientPublicKey length is incorrect
 * @throws when senderSecretKey is not an `Uint8Array`
 * @throws when senderSecretKey length is incorrect
 * @throws when the encryption didn't succeed successfully
 */
function boxSeal(message, recipientPublicKey, senderSecretKey) {
  const nonceBuffer = (0, _tweetnacl.randomBytes)(_tweetnacl.box.nonceLength);
  const cipherTextBuffer = (0, _tweetnacl.box)(message, nonceBuffer, recipientPublicKey, senderSecretKey);
  const encrypted = new Uint8Array(nonceBuffer.length + cipherTextBuffer.length);
  encrypted.set(nonceBuffer);
  encrypted.set(cipherTextBuffer);
  return encrypted;
}

/**
 * Verifies and decrypts a ciphertext produced by `boxSeal`
 *
 * @throws when cipherText is not an `Uint8Array`
 * @throws when senderPublicKey is not an `Uint8Array`
 * @throws when senderPublicKey length is incorrect
 * @throws when recipientSecretKey is not an `Uint8Array`
 * @throws when recipientSecretKey length is incorrect
 * @throws when the verification failed
 */
function boxOpen(cipherText, senderPublicKey, recipientSecretKey) {
  const nonceBuffer = cipherText.slice(0, _tweetnacl.box.nonceLength);
  const decrypted = _tweetnacl.box.open(cipherText.slice(_tweetnacl.box.nonceLength), nonceBuffer, senderPublicKey, recipientSecretKey);
  if (decrypted === null) {
    throw new Error('[react-native-nacl-jsi] boxOpen verification failed');
  }
  return decrypted;
}

/**
 * Generates a key to be used with secretboxSeal and secretboxOpen
 */
function secretboxGenerateKey() {
  return (0, _tweetnacl.randomBytes)(_tweetnacl.secretbox.keyLength);
}

/**
 * Encrypts a message with the given secret key
 *
 * @throws when message is not a `Uint8Array`
 * @throws when secretKey is not a `Uint8Array`
 * @throws when secretKey length is incorrect
 * @throws when the encryption didn't complete successfully
 */
function secretboxSeal(message, secretKey) {
  const nonceBuffer = (0, _tweetnacl.randomBytes)(_tweetnacl.secretbox.nonceLength);
  const cipherTextBuffer = (0, _tweetnacl.secretbox)(message, nonceBuffer, secretKey);
  const encrypted = new Uint8Array(nonceBuffer.length + cipherTextBuffer.length);
  encrypted.set(nonceBuffer);
  encrypted.set(cipherTextBuffer);
  return encrypted;
}

/**
 * Verifies and decrypts a ciphertext produced by `secretboxSeal`
 *
 * @throws when cipherText is not an `Uint8Array`
 * @throws when secretKey is not an `Uint8Array`
 * @throws when secretKey length is incorrect
 * @throws when the verification failed
 */
function secretboxOpen(cipherText, secretKey) {
  const decrypted = _tweetnacl.secretbox.open(cipherText.slice(_tweetnacl.secretbox.nonceLength), cipherText.slice(0, _tweetnacl.secretbox.nonceLength), secretKey);
  if (decrypted === null) {
    throw new Error('[react-native-nacl-jsi] secretboxOpen verification failed');
  }
  return decrypted;
}

/**
 * Generates a key pair for signing and verifying signatures
 */
function signGenerateKey() {
  return _tweetnacl.sign.keyPair();
}

/**
 * Signs a message with the given secret key
 *
 * @throws when message is not a `Uint8Array`
 * @throws when secretKey is not a `Uint8Array`
 * @throws when secretKey length is incorrect
 */
function signDetached(message, secretKey) {
  const keyPair = _tweetnacl.sign.keyPair.fromSecretKey(secretKey);
  return _tweetnacl.sign.detached(message, keyPair.publicKey);
}

/**
 * Verifies that the signature is valid given the message the public key
 *
 * @throws when message is not a `Uint8Array`
 * @throws when publicKey is not a `Uint8Array`
 * @throws when publicKey length is incorrect
 * @throws when signature is not a `Uint8Array`
 */
function signVerifyDetached(message, publicKey, signature) {
  return _tweetnacl.sign.detached.verify(message, signature, publicKey);
}

/**
 * Hash a password using the Argon2id algorithm
 *
 * @throws when password is not a `Uint8Array`
 * @throws when the hash computation didn't complete successfully
 */
async function argon2idHash(password, iterations, memoryLimit) {
  const salt = (0, _tweetnacl.randomBytes)(16);
  const key = await (0, _hashWasm.argon2id)({
    password,
    salt,
    parallelism: 1,
    iterations: Number(iterations),
    memorySize: Number(memoryLimit),
    hashLength: 32,
    outputType: 'encoded'
  });
  return key;
}

/**
 * Verifies that `hash` is a valid `password` verification string
 *
 * @throws when password is not a `Uint8Array`
 */
async function argon2idVerify(hash, password) {
  const isVerified = await (0, _hashWasm.argon2Verify)({
    password,
    hash
  });
  return isVerified;
}

/**
 * Derives a key of the given length
 *
 * @throws when password is not a `Uint8Array`
 * @throws when salt is not a `Uint8Array` or its size is not `ARGON2ID_SALTBYTES`
 */
async function argon2idDeriveKey(password, salt, keyLength, iterations, memoryLimit) {
  const derivedKey = await (0, _hashWasm.argon2id)({
    password,
    salt,
    parallelism: 1,
    hashLength: keyLength,
    iterations: Number(iterations),
    memorySize: Number(memoryLimit),
    outputType: 'binary'
  });
  return derivedKey;
}

/**
 * Returns a buffer of the given size filled with random bytes
 *
 * @throws When the given size is incorrect
 */
function getRandomBytes(size) {
  return (0, _tweetnacl.randomBytes)(typeof size === 'number' ? size : Number(size));
}
//# sourceMappingURL=nacl.web.js.map